# -*- coding: utf-8 -*-

"""Top-level package for Python RSI."""
__author__ = """Ventorvar"""
__email__ = 'ventorvar@gmail.com'
__version__ = '0.1.16'



# TODO: future endpoints to check out
# https://robertsspaceindustries.com/api/stats/getCrowdfundStats
# https://robertsspaceindustries.com/api/account/badge/getBadges

from .rsi import RSISite
